/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.kernel.handlers.HotkeyHandler;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.kernel.utils.SystemProperties.OperatingSystem;
import net.sourceforge.atunes.utils.LanguageTool;

import org.jvnet.substance.api.renderers.SubstanceDefaultTableCellRenderer;

/**
 * The Class PlayerPanel.
 */
public class PlayerPanel extends PreferencesPanel {

	/**
	 * The Class HotkeyTableModel.
	 */
	private static class HotkeyTableModel extends AbstractTableModel {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = -5726677745418003289L;

		/** The data. */
		private String[][] data;

		/**
		 * Instantiates a new hotkey table model.
		 */
		public HotkeyTableModel() {
			refreshData();
		}

		/**
		 * Refresh data.
		 */
		public void refreshData() {
			data = HotkeyHandler.getInstance().getHotkeyDescription();
			fireTableDataChanged();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		@Override
		public int getColumnCount() {
			return 2;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
		 */
		@Override
		public String getColumnName(int column) {
			return column == 0 ? LanguageTool.getString("HOTKEY") : LanguageTool.getString("ACTION");
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		@Override
		public int getRowCount() {
			return data != null ? data.length : 0;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			return data != null ? data[rowIndex][columnIndex] : "";
		}

	}

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 4489293347321979288L;

	/** The play at startup. */
	private JCheckBox playAtStartup;

	/** The show ticks. */
	private JCheckBox showTicks;

	/** The use fade away. */
	private JCheckBox useFadeAway;
	
	/** The use short path names. */
	private JCheckBox useShortPathNames;

	/** The enable global hotkeys. */
	private JCheckBox enableGlobalHotkeys;

	/** The hotkey table. */
	private JTable hotkeyTable;

	/** The table model. */
	private HotkeyTableModel tableModel = new HotkeyTableModel();

	/** The cache files before playing. */
	private JCheckBox cacheFilesBeforePlaying;

	private JComboBox engineCombo;

	/**
	 * Instantiates a new player panel.
	 */
	public PlayerPanel() {
		super();
		Box engineBox = Box.createHorizontalBox();
		engineBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		engineBox.add(new JLabel(LanguageTool.getString("PLAYER_ENGINE")));
		engineBox.add(Box.createHorizontalStrut(6));
		engineCombo = new JComboBox(PlayerHandler.getInstance().getEngineNames());
		engineBox.add(engineCombo);
		engineBox.add(Box.createHorizontalGlue());
		playAtStartup = new JCheckBox(LanguageTool.getString("PLAY_AT_STARTUP"));
		useFadeAway = new JCheckBox(LanguageTool.getString("USE_FADE_AWAY"));	
		showTicks = new JCheckBox(LanguageTool.getString("SHOW_TICKS"));
		useShortPathNames = new JCheckBox(LanguageTool.getString("USE_SHORT_PATH_NAMES_FOR_MPLAYER"));
		enableGlobalHotkeys = new JCheckBox(LanguageTool.getString("ENABLE_GLOBAL_HOTKEYS"));
		hotkeyTable = new JTable();
		hotkeyTable.setModel(tableModel);
		hotkeyTable.getTableHeader().setReorderingAllowed(false);
		hotkeyTable.getTableHeader().setResizingAllowed(false);
		hotkeyTable.setEnabled(HotkeyHandler.areHotkeysAndIntellitypeSupported());
		hotkeyTable.setDefaultRenderer(Object.class, new SubstanceDefaultTableCellRenderer() {
			private static final long serialVersionUID = 1111298953883261220L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean arg2, boolean arg3, int arg4, int arg5) {
				Component c = super.getTableCellRendererComponent(table, value, arg2, arg3, arg4, arg5);

				GuiUtils.applyComponentOrientation((JLabel) c);

				return c;
			}
		});
		JScrollPane scrollPane = new JScrollPane(hotkeyTable);
		scrollPane.setMinimumSize(new Dimension(400, 200));
		cacheFilesBeforePlaying = new JCheckBox(LanguageTool.getString("CACHE_FILES_BEFORE_PLAYING"));

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		Insets prevInsets = c.insets;
		c.insets = new Insets(0, 8, 0, 0);
		add(engineBox, c);
		c.insets = prevInsets;
		c.gridy = 1;
		add(playAtStartup, c);
		c.gridy = 2;
		add(useFadeAway, c);
		c.gridy = 3;
		add(showTicks, c);
		c.gridy = 4;
		add(useShortPathNames, c);
		c.gridy = 5;
		add(cacheFilesBeforePlaying, c);
		c.gridy = 6;
		add(enableGlobalHotkeys, c);
		c.gridy = 7;
		c.weighty = 1;
		c.weightx = 0;
		c.insets = new Insets(10, 20, 10, 10);
		add(scrollPane, c);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#setVisible(boolean)
	 */
	@Override
	public void setVisible(boolean flag) {
		if (flag) {
			tableModel.refreshData();
		}
		super.setVisible(flag);
	}

	/**
	 * Gets the enable global hotkeys.
	 * 
	 * @return the enable global hotkeys
	 */
	public JCheckBox getEnableGlobalHotkeys() {
		return enableGlobalHotkeys;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.views.dialogs.editPreferences.PreferencesPanel
	 * #getResult()
	 */

	@Override
	public Map<String, Object> getResult() {
		Map<String, Object> result = new HashMap<String, Object>();
		result.put("PLAY_AT_STARTUP", playAtStartup.isSelected());
		result.put("USE_FADE_AWAY", useFadeAway.isSelected());
		result.put("SHOW_TICKS", showTicks.isSelected());
		result.put("USE_SHORT_PATH_NAMES_FOR_MPLAYER", useShortPathNames.isSelected());
		result.put("ENABLE_GOLBAL_HOTKEYS", enableGlobalHotkeys.isSelected());
		result.put("CACHE_FILES_BEFORE_PLAYING", cacheFilesBeforePlaying.isSelected());
		result.put("PLAYER_ENGINE", engineCombo.getSelectedItem().toString());
		return result;
	}

	/**
	 * Gets the use short path names.
	 * 
	 * @return the use short path names
	 */
	private JCheckBox getUseShortPathNames() {
		return useShortPathNames;
	}

	/**
	 * Sets the enable hotkeys.
	 * 
	 * @param enable
	 *            the new enable hotkeys
	 */
	private void setEnableHotkeys(boolean enable) {
		enableGlobalHotkeys.setSelected(enable);
	}

	/**
	 * Sets the play at startup.
	 * 
	 * @param play
	 *            the new play at startup
	 */
	private void setPlayAtStartup(boolean play) {
		playAtStartup.setSelected(play);
	}

	/**
	 * Sets the use short path names.
	 * 
	 * @param use
	 *            the new use short path names
	 */
	private void setUseShortPathNames(boolean use) {
		useShortPathNames.setSelected(use);
	}

	/**
	 * Sets the use fade away.
	 * 
	 * @param useFadeAway
	 *            the new use fade away
	 */
	private void setUseFadeAway(boolean useFadeAway) {
		this.useFadeAway.setSelected(useFadeAway);
	}
	
	/**
	 * Sets the show ticks.
	 * 
	 * @param showTicks
	 *            the new show ticks
	 */
	private void setShowTicks(boolean showTicks) {
		this.showTicks.setSelected(showTicks);
	}
	
	/**
	 * Sets the cache files before playing.
	 * 
	 * @param cacheFiles
	 *            the new cache files before playing
	 */
	private void setCacheFilesBeforePlaying(boolean cacheFiles) {
		this.cacheFilesBeforePlaying.setSelected(cacheFiles);
	}

	private void setPlayerEngine(String playerEngine) {
		engineCombo.setSelectedItem(playerEngine);
	}

	@Override
	public void updatePanel(ApplicationState state) {
		setPlayerEngine(state.getPlayerEngine());
		setPlayAtStartup(state.isPlayAtStartup());
		setUseFadeAway(state.isUseFadeAway());
		setShowTicks(state.isShowTicks());
		setCacheFilesBeforePlaying(state.isCacheFilesBeforePlaying());
		setEnableHotkeys(state.isEnableHotkeys());
		setUseShortPathNames(state.isUseShortPathNames());
		getUseShortPathNames().setEnabled(SystemProperties.OS == OperatingSystem.WINDOWS);
		getEnableGlobalHotkeys().setEnabled(HotkeyHandler.areHotkeysAndIntellitypeSupported());
	}
}
