/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.ApplicationArguments;
import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.ColorDefinitions;
import net.sourceforge.atunes.gui.Fonts;
import net.sourceforge.atunes.gui.LookAndFeelSelector;
import net.sourceforge.atunes.kernel.handlers.ApplicationDataHandler;
import net.sourceforge.atunes.kernel.handlers.ContextInformationHandler;
import net.sourceforge.atunes.kernel.handlers.DeviceHandler;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.handlers.MultipleInstancesHandler;
import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.handlers.PodcastFeedHandler;
import net.sourceforge.atunes.kernel.handlers.RadioHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.handlers.UpdateHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionMonitor;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListIO;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.kernel.utils.LanguageSelector;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.Timer;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Kernel is the class responsible of create and interconnect all modules of
 * aTunes.
 */
public class Kernel {

	/** Logger. */
	private static Logger logger = new Logger();

	/**
	 * Unique instance of Kernel. To access Kernel, Kernel.getInstance() must be
	 * called
	 */
	static Kernel instance;

	/** Defines if aTunes is running in debug mode. */
	public static boolean DEBUG;

	/** Defines if aTunes will ignore look and feel. */
	public static boolean IGNORE_LOOK_AND_FEEL;

	/** Defines if aTunes should not try to update (for Linux packages). */
	public static boolean NO_UPDATE;

	/** Application State of aTunes. */
	public ApplicationState state;

	/**
	 * List of finish listeners
	 */
	private List<ApplicationFinishListener> finishListeners;

	/**
	 * Constructor of Kernel.
	 */
	protected Kernel() {
		finishListeners = new ArrayList<ApplicationFinishListener>();
	}

	/**
	 * Getter of the Kernel instance.
	 * 
	 * @return Kernel
	 */
	public static Kernel getInstance() {
		return instance;
	}

	/**
	 * Adds a finish listener to list of listeners. All classes that implements
	 * ApplicationFinishListener must call this method in order to be notified
	 * by Kernel when application is finishing
	 * 
	 * @param listener
	 */
	public void addFinishListener(ApplicationFinishListener listener) {
		this.finishListeners.add(listener);
	}

	/**
	 * Static method to create the Kernel instance. This method starts the
	 * application, so should be called from the main method of the application.
	 * 
	 * @param args
	 *            the args
	 */
	public static void startKernel(final List<String> args) {
		logger.debug(LogCategories.START, "Starting Kernel");

		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					instance = new Kernel();
					if (!ApplicationDataHandler.getInstance().readState()) {
						instance.state = new ApplicationState();
					}

					// Add application finish listener
					Kernel.getInstance().addFinishListener(MultipleInstancesHandler.getInstance());
					// Set font smoothing
					Fonts.setFontSmoothing();
					// Set look and feel
					LookAndFeelSelector.setLookAndFeel(instance.state.getSkin());
					ColorDefinitions.initColors();
					// Set language
					LanguageSelector.setLanguage();
					// Init fonts
					Fonts.initializeFonts();
				}
			});
		} catch (Exception e) {
			logger.error(LogCategories.START, e);
			logger.error(LogCategories.START, e.getCause());
		}

		// Show title dialog
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					VisualHandler.getInstance().showSplashScreen();
				}
			});
		} catch (Exception e) {
			logger.error(LogCategories.START, e);
			logger.error(LogCategories.START, e.getCause());
		}

		// Read caches
		Scheduler.runInParallel(true, RadioHandler.getInstance().getReadRadiosRunnable(), PodcastFeedHandler.getInstance().getReadPodcastFeedsRunnable(), FavoritesHandler
				.getInstance().getReadFavotiresRunnable(), PlayListHandler.getInstance().getReadPlayListsRunnable(), RepositoryHandler.getInstance()
				.getReadRepositoryFromCacheRunnable());

		// Find for audio files on arguments
		final List<String> songs = new ArrayList<String>();
		for (String arg : args) {
			if (AudioFile.isValidAudioFile(arg)) {
				songs.add(arg);
			} else if (PlayListIO.isValidPlayList(arg)) {
				songs.addAll(PlayListIO.read(new File(arg)));
			}
		}

		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					// Start component creation
					instance.startCreation();
					// Start bussiness
					instance.start(PlayListIO.getAudioObjectsFromFileNamesList(songs));
				}
			});
		} catch (Exception e) {
			logger.error(LogCategories.START, e);
			logger.error(LogCategories.START, e.getCause());
		}

	}

	/**
	 * Executes actions needed before closing application, finished all
	 * necessary modules and writes configuration.
	 */
	private void callActionsBeforeEnd() {
		// Call all ApplicationFinishListener instances to finish
		for (ApplicationFinishListener finishListener : finishListeners) {
			try {
				finishListener.applicationFinish();
			} catch (Exception e) {
				logger.error(LogCategories.END, e);
			}
		}
	}

	/**
	 * Called when closing application
	 */
	public void finish() {
		try {
			Timer.start();
			logger.info(LogCategories.END, StringUtils.getString("Closing ", Constants.APP_NAME, " ", Constants.VERSION.toString()));
			// Stop must be called explicitely to avoid playback after user closed app
			PlayerHandler.getInstance().stopCurrentAudioObject(true);
			// Call actions needed before closing application
			callActionsBeforeEnd();
		} finally {
			logger.info(LogCategories.END, StringUtils.getString("Application finished (", StringUtils.toString(Timer.stop(), 3), " seconds)"));
			logger.info(LogCategories.END, "Goodbye!!");
			// Exit normally
			System.exit(0);
		}
	}

	/**
	 * Once all application is loaded, it's time to load data (repository,
	 * playlist, ...)
	 * 
	 * @param playList
	 *            the play list
	 */
	void start(List<AudioObject> playList) {
		try {
			Scheduler.runInParallel(true, ApplicationDataHandler.getInstance().getApplyStateRunnable());

			RepositoryHandler.getInstance().setRepository();

			PlayListHandler.getInstance().setPlayLists();
			if (!playList.isEmpty()) {
				PlayListHandler.getInstance().addToPlayListAndPlay(playList);
				ControllerProxy.getInstance().getPlayListController().refreshPlayList();
			}
		} catch (Exception e) {
			logger.error(LogCategories.KERNEL, e);
		}
		VisualHandler.getInstance().setFullFrameVisible(true);
		//Hide title dialog
		VisualHandler.getInstance().hideSplashScreen();

		logger.info(LogCategories.START, StringUtils.getString("Application started (", StringUtils.toString(Timer.stop(), 3), " seconds)"));

		callActionsAfterStart();
	}

	/**
	 * Call actions after start.
	 */
	private void callActionsAfterStart() {

		if (state.isPlayAtStartup()) {
			Scheduler.scheduleTaskAfter("Start Play", PlayListHandler.getInstance().getStartToPlayRunnable(), 3);
		}

		Scheduler.scheduleTaskAfter("Repository Post Init Actions", RepositoryHandler.getInstance().getAfterStartActionsRunnable(), 3);
		Scheduler.scheduleTaskAfter("Favorite Post Init Actions", FavoritesHandler.getInstance().getAfterStartActionsRunnable(), 3);

		ContextInformationHandler.getInstance().submitCacheToLastFm();

		Scheduler.scheduleTaskAfter("Device Monitor", new Runnable() {
			@Override
			public void run() {
				// Start device monitor
				DeviceConnectionMonitor.startMonitor();
				DeviceConnectionMonitor.addListener(DeviceHandler.getInstance());
			}
		}, 10);

		Scheduler.scheduleTaskAfter("Podcast Retriever", new Runnable() {
			@Override
			public void run() {
				// Start podcast retriever
				PodcastFeedHandler.getInstance().startPodcastFeedEntryDownloadChecker();
				PodcastFeedHandler.getInstance().startPodcastFeedEntryRetriever();
			}
		}, 15);
		if (!NO_UPDATE) {
			Scheduler.scheduleTaskAfter("Check Updates", new Runnable() {
				@Override
				public void run() {
					// Check for updates
					UpdateHandler.getInstance().checkUpdates(state.getProxy(), false, false);
				}
			}, 20);
		}
	}

	/**
	 * Starts controllers associated to visual classes.
	 */
	private void startControllers() {
		ControllerProxy.getInstance();
	}

	/**
	 * Creates all objects of aTunes: visual objects, controllers, and handlers.
	 */
	void startCreation() {
		logger.debug(LogCategories.START, "Starting components");

		startVisualization();
		startControllers();
		VisualHandler.getInstance().setTitleBar("");
	}

	/**
	 * Starts visual objects.
	 */
	private void startVisualization() {
		VisualHandler.getInstance().startVisualization();
	}

	/**
	 * Called when restarting application
	 */
	public void restart() {
		try {
			// Store all configuration and finish all active modules
			callActionsBeforeEnd();

			// Build a process builder with OS-specific command and saved arguments
			ProcessBuilder pb = new ProcessBuilder(SystemProperties.OS.getLaunchCommand(), ApplicationArguments.getSavedArguments());

			System.out.println(pb.command().toString());

			// Start new application instance
			pb.start();

		} catch (IOException e) {
			logger.error(LogCategories.KERNEL, e);
		} finally {
			// Exit normally
			System.exit(0);
		}
	}
}
