/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.exportOptionsDialog;

import java.io.File;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import net.sourceforge.atunes.gui.views.dialogs.ExportOptionsDialog;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.controllers.model.DialogController;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.process.ProcessListener;
import net.sourceforge.atunes.kernel.modules.repository.ExportFilesProcess;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.log4j.LogCategories;

/**
 * The Class ExportOptionsDialogController.
 */
public class ExportOptionsDialogController extends DialogController<ExportOptionsDialog> {

	/** The export type. */
	private int exportType;

	/** The export structure. */
	private int exportStructure;

	/** The file pattern. */
	private String filePattern;

	/** The path. */
	private String path;

	/** The canceled. */
	private boolean canceled;

	/** The dialog controlled. */
	private ExportOptionsDialog dialogControlled;

	/**
	 * Instantiates a new export options dialog controller.
	 * 
	 * @param dialogControlled
	 *            the dialog controlled
	 */
	public ExportOptionsDialogController(ExportOptionsDialog dialogControlled) {
		super(dialogControlled);
		this.dialogControlled = dialogControlled;
		addBindings();
		addStateBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		ExportOptionsDialogListener listener = new ExportOptionsDialogListener(this, dialogControlled);
		dialogControlled.getExportButton().addActionListener(listener);
		dialogControlled.getCancelButton().addActionListener(listener);
		dialogControlled.getFlatStructureRadioButton().addActionListener(listener);
		dialogControlled.getArtistStructureRadioButton().addActionListener(listener);
		dialogControlled.getFullStructureRadioButton().addActionListener(listener);
		dialogControlled.getNoChangeFileNamesRadioButton().addActionListener(listener);
		dialogControlled.getAllTypeRadioButton().addActionListener(listener);
		dialogControlled.getSelectionTypeRadioButton().addActionListener(listener);
		dialogControlled.getFavoritesTypeRadioButton().addActionListener(listener);
		dialogControlled.addWindowListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		exportType = ExportFilesProcess.ALL_EXPORT;
		dialogControlled.getAllTypeRadioButton().setSelected(true);

		exportStructure = ExportFilesProcess.FULL_STRUCTURE;
		dialogControlled.getFullStructureRadioButton().setSelected(true);

		filePattern = null;
		dialogControlled.getNoChangeFileNamesRadioButton().setSelected(true);
	}

	/**
	 * Begin export process.
	 */

	public void beginExportProcess() {
		getLogger().debug(LogCategories.CONTROLLER);

		dialogControlled.setVisible(true);
		if (!canceled) {
			boolean isValidPath = verifyPath();
			boolean isValidFilePattern = verifyFilePattern();
			if (isValidPath && isValidFilePattern) {
				boolean pathExists = new File(path).exists();
				boolean userWantsToCreate = false;
				if (!pathExists) {
					if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("DIR_NO_EXISTS"), LanguageTool.getString("INFO")) == JOptionPane.OK_OPTION) {
						pathExists = new File(path).mkdir();
						userWantsToCreate = true;
					}
				}
				if (pathExists) {
					List<AudioFile> songs;
					if (exportType == ExportFilesProcess.ALL_EXPORT) {
						songs = RepositoryHandler.getInstance().getAudioFiles();
					} else if (exportType == ExportFilesProcess.SELECTION_EXPORT) {
						songs = AudioFile.getAudioFiles(ControllerProxy.getInstance().getNavigationController().getSongsForNavigationTree());
					} else {
						songs = FavoritesHandler.getInstance().getFavoriteSongs();
					}

					ExportFilesProcess process = new ExportFilesProcess(songs, exportStructure, filePattern, path);
					process.addProcessListener(new ProcessListener() {
						@Override
						public void processCanceled() { /* Nothing to do */ }
						
						@Override
						public void processFinished(final boolean ok) {
							SwingUtilities.invokeLater(new Runnable() {
								@Override
								public void run() {
									if (!ok) {
										VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("ERRORS_IN_EXPORT_PROCESS"));	
									}
								}
							});
						}
					});
					process.execute();
				} else if (userWantsToCreate) {
					VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("COULD_NOT_CREATE_DIR"));
				}
			} else {
				if (!isValidPath) {
					VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("INCORRECT_EXPORT_PATH"));
				}
				if (!isValidFilePattern) {
					VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("INCORRECT_FILE_PATTERN"));
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Sets the canceled.
	 * 
	 * @param canceled
	 *            the new canceled
	 */
	public void setCanceled(boolean canceled) {
		this.canceled = canceled;
	}

	/**
	 * Sets the export structure.
	 * 
	 * @param exportStructure
	 *            the new export structure
	 */
	public void setExportStructure(int exportStructure) {
		this.exportStructure = exportStructure;
	}

	/**
	 * Sets the export type.
	 * 
	 * @param exportType
	 *            the new export type
	 */
	public void setExportType(int exportType) {
		this.exportType = exportType;
	}

	/**
	 * Sets the file pattern.
	 * 
	 * @param filePattern
	 *            the new file pattern
	 */
	public void setFilePattern(String filePattern) {
		this.filePattern = filePattern;
	}

	/**
	 * Sets the path.
	 * 
	 * @param path
	 *            the new path
	 */
	public void setPath(String path) {
		this.path = path;
	}

	/**
	 * Verify file pattern.
	 * 
	 * @return true, if successful
	 */
	private boolean verifyFilePattern() {
		return filePattern == null || !filePattern.equals("");
	}

	/**
	 * Verify path.
	 * 
	 * @return true, if successful
	 */
	private boolean verifyPath() {
		return path != null && !path.equals("");
	}
}
