/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.playerControls;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.panels.PlayerControlsPanel;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.handlers.SystemTrayHandler;

import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.audio.CueTrack;
import net.sourceforge.atunes.model.AudioObject;

/**
 * The listener interface for receiving playerControls events.
 */
public class PlayerControlsListener extends MouseAdapter implements ActionListener, ChangeListener, MouseWheelListener {

	/** The panel. */
	private PlayerControlsPanel panel;

	/** The controller. */
	private PlayerControlsController controller;

	/**
	 * Instantiates a new player controls listener.
	 * 
	 * @param panel
	 *            the panel
	 * @param controller
	 *            the controller
	 */
	protected PlayerControlsListener(PlayerControlsPanel panel, PlayerControlsController controller) {
		this.panel = panel;
		this.controller = controller;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(panel.getVolumeButton())) {
			boolean mute = !Kernel.getInstance().state.isMuteEnabled();
			Kernel.getInstance().state.setMuteEnabled(mute);
			controller.setMute(mute);
			SystemTrayHandler.getInstance().setMute(mute);
			PlayerHandler.getInstance().applyMuteState(mute);
		} else if (e.getSource().equals(panel.getPlayButton())) {
			PlayerHandler.getInstance().playCurrentAudioObject(true);
		} else if (e.getSource().equals(panel.getStopButton())) {
			PlayerHandler.getInstance().stopCurrentAudioObject(true);
		} else if (e.getSource().equals(panel.getNextButton())) {
			PlayerHandler.getInstance().playNextAudioObject(false);
		} else if (e.getSource().equals(panel.getPreviousButton())) {
			PlayerHandler.getInstance().playPreviousAudioObject();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseReleased(MouseEvent e) {
		if (e.getSource().equals(panel.getProgressBar()) && panel.getProgressBar().isEnabled()) {
			//int value = ((JSlider)e.getSource()).getValue();
			//double perCent = (double) value / ((JSlider)e.getSource()).getMaximum();
			//double perCentOfSong = value * 1000000 / duration;
			int widthClicked;
			if (GuiUtils.getComponentOrientation().isLeftToRight()) {
				widthClicked = e.getPoint().x;
			} else {
				widthClicked = ((JSlider) e.getSource()).getWidth() - e.getPoint().x;
			}
			int widthOfProgressBar = panel.getProgressBar().getSize().width;
			double perCent = (double) widthClicked / (double) widthOfProgressBar;
			AudioObject audioObject = PlayListHandler.getInstance().getCurrentPlayList(false).getCurrentAudioObject();
			// If it is a cue track, then adjust the percent
			if (AudioFile.isValidAudioFile(audioObject.getUrl().toString())) {	
				if (AudioFile.isCueFile(((AudioFile) audioObject).getFile())) {
					int startPosition = ((CueTrack) audioObject).getTrackStartPositionAsInt();
					float duration = ((CueTrack) audioObject).getDuration();
					long trackDuration = ((CueTrack) audioObject).getTotalDuration();
					perCent = (startPosition + perCent * duration) / trackDuration;
				}
			}
			PlayerHandler.getInstance().seekCurrentAudioObject(perCent);
		}
	}

	@Override
	public void stateChanged(ChangeEvent e) {

		if (e.getSource().equals(panel.getVolumeSlider())) {
			applyVolumeChange();

		} else if (e.getSource().equals(panel.getBalanceSlider())) {
			int value = ((JSlider) e.getSource()).getValue();
			if (value > 4) {
				panel.getBalanceLabel().setIcon(ImageLoader.BALANCE_RIGHT);
			} else if (value > 0) {
				panel.getBalanceLabel().setIcon(ImageLoader.BALANCE_MED_RIGHT);
			} else if (value == 0) {
				panel.getBalanceLabel().setIcon(ImageLoader.BALANCE_MED);
			} else if (value > -4) {
				panel.getBalanceLabel().setIcon(ImageLoader.BALANCE_MED_LEFT);
			} else {
				panel.getBalanceLabel().setIcon(ImageLoader.BALANCE_LEFT);
			}
			// TODO: Remove this code, balance is no more used
		}
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {

		if (e.getSource().equals(panel.getVolumeSlider())) {

			JSlider volumeSlider = panel.getVolumeSlider();

			int notches = e.getWheelRotation();
			if (notches < 0) {
				volumeSlider.setValue(volumeSlider.getValue() + 1);
			} else {
				volumeSlider.setValue(volumeSlider.getValue() - 1);
			}

			applyVolumeChange();
		}
	}

	/**
	 * Apply a changed volume value
	 */
	private void applyVolumeChange() {

		int value = panel.getVolumeSlider().getValue();
		if (!panel.getVolumeButton().isSelected()) {
			if (value > 80) {
				panel.getVolumeButton().setIcon(ImageLoader.VOlUME_MAX);
			} else if (value > 40) {
				panel.getVolumeButton().setIcon(ImageLoader.VOLUME_MED);
			} else if (value > 5) {
				panel.getVolumeButton().setIcon(ImageLoader.VOLUME_MIN);
			} else {
				panel.getVolumeButton().setIcon(ImageLoader.VOLUME_ZERO);
			}
		}
		Kernel.getInstance().state.setVolume(value);
		PlayerHandler.getInstance().setVolume(value);
		controller.setVolume(value);
	}

}
