/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.views.dialogs.FileSelectionDialog;
import net.sourceforge.atunes.kernel.ApplicationFinishListener;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.ConnectDeviceAction;
import net.sourceforge.atunes.kernel.actions.DisconnectDeviceAction;
import net.sourceforge.atunes.kernel.actions.RefreshDeviceAction;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionListener;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionMonitor;
import net.sourceforge.atunes.kernel.modules.device.DeviceDisconnectionListener;
import net.sourceforge.atunes.kernel.modules.device.DeviceDisconnectionMonitor;
import net.sourceforge.atunes.kernel.modules.device.TransferToDeviceProcess;
import net.sourceforge.atunes.kernel.modules.navigator.DeviceNavigationView;
import net.sourceforge.atunes.kernel.modules.process.ProcessListener;
import net.sourceforge.atunes.kernel.modules.repository.ExportFilesProcess;
import net.sourceforge.atunes.kernel.modules.repository.LoaderListener;
import net.sourceforge.atunes.kernel.modules.repository.Repository;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.audio.CueTrack;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.kernel.modules.search.searchableobjects.DeviceSearchableObject;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class DeviceHandler.
 */
public class DeviceHandler implements LoaderListener, DeviceConnectionListener, DeviceDisconnectionListener, ApplicationFinishListener {

	/** The instance. */
	private static DeviceHandler instance = new DeviceHandler();

	/** The logger. */
	private Logger logger = new Logger();

	/** The device repository. */
	private Repository deviceRepository;

	/** The temp device repository. */
	private Repository tempDeviceRepository;

	/** The current loader. */
	private RepositoryLoader currentLoader;

	/** The device path. */
	private File devicePath;

	/**
	 * Device identification
	 */
	private String deviceId;

	/** The state. */
	private ApplicationState state = Kernel.getInstance().state;

	/**
	 * Instantiates a new device handler.
	 */
	private DeviceHandler() {
		Kernel.getInstance().addFinishListener(this);
	}

	/**
	 * Gets the single instance of DeviceHandler.
	 * 
	 * @return single instance of DeviceHandler
	 */
	public static DeviceHandler getInstance() {
		return instance;
	}

	/**
	 * Fills the device with songs until the specified memory is left.
	 * 
	 * @param leaveFreeLong
	 *            Memory to leave free
	 */
	public void fillWithRandomSongs(long leaveFreeLong) {
		long leaveFree = leaveFreeLong;
		logger.debug(LogCategories.HANDLER, new String[] { Long.toString(leaveFree) });

		// Get reference to Repository songs
		List<AudioFile> songs = RepositoryHandler.getInstance().getAudioFiles();

		// Songs selected
		List<AudioFile> songsSelected = new ArrayList<AudioFile>();

		// Initialize random generator
		Random r = new Random(new Date().getTime());

		// Get free space in device
		long deviceFreeSpace = deviceRepository.getFolders().get(0).getFreeSpace();
		if (leaveFree >= 0) {
			leaveFree = (leaveFree) * 1048576; // Transfrom to same unit
		} else {
			leaveFree = 0; // User typed nonsense value. So fill his device ...
		}

		// Not enough space avaible
		if (leaveFree > deviceFreeSpace) {
			VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("NOT_ENOUGH_SPACE_ON_DEVICE"));
			return;
		}

		// Get n songs
		for (int i = 0; leaveFree < deviceFreeSpace; i++) {
			// Get song number
			int number = r.nextInt(songs.size());

			// Add selectedSong
			songsSelected.add(songs.get(number));

			// Compute free space after song has been added
			deviceFreeSpace = deviceFreeSpace - songs.get(number).getFile().length();
		}

		// Write files
		copyFilesToDevice(songsSelected);
	}

	/**
	 * Connect device.
	 */
	public void connectDevice() {
		FileSelectionDialog dialog = VisualHandler.getInstance().getFileSelectionDialog(true);
		dialog.setTitle(LanguageTool.getString("SELECT_DEVICE"));
		dialog.startDialog();
		if (!dialog.isCanceled()) {
			File dir = dialog.getSelectedDir();

			VisualHandler.getInstance().showProgressBar(true, null);
			this.retrieveDevice(dir);
		}
	}

	/**
	 * Copy files to mp3 device.
	 * 
	 * @param files
	 *            Files to be written to a mp3 device
	 */

	public void copyFilesToDevice(List<AudioFile> files) {
		// Get size of files
		long size = 0;
		for (AudioFile file : files) {
			if(file instanceof CueTrack){
				//if file is a cue track
				AudioFile audiofile = ((CueTrack)file).getCueFile();
				//add its audio file into files list
				if(!files.contains(audiofile)){
					files.add(audiofile);
				}
				//remove cue track from files list
				files.remove(file);
				continue;
			}
			size = size + file.getFile().length();
		}
		// Get free space in device
		long deviceFreeSpace = deviceRepository.getFolders().get(0).getFreeSpace();

		// Get folder structure to use
		int structure;
		if (state.isFlatStructure()) {
			structure = ExportFilesProcess.FLAT_STRUCTURE;
		} else if (state.isArtistStructure()) {
			structure = ExportFilesProcess.ARTIST_STRUCTURE;
		} else {
			structure = ExportFilesProcess.FULL_STRUCTURE;
		}

		// Check if there is enough free space on device
		if (size > deviceFreeSpace) {
			VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("NOT_ENOUGH_SPACE_ON_DEVICE"));
			return;
		}

		TransferToDeviceProcess process = new TransferToDeviceProcess(files, structure, deviceRepository.getFolders().get(0).getAbsolutePath());
		process.addProcessListener(new ProcessListener() {
			@Override
			public void processCanceled() {}

			@Override
			public void processFinished(final boolean ok) {
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						refreshDevice();
						if (!ok) {
							VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("ERRORS_IN_EXPORT_PROCESS"));
						}
					}
				});
			}
		});
		process.execute();
	}

	/**
	 * Called when a device monitor detects a device connected.
	 * 
	 * @param location
	 *            the location
	 */
	@Override
	public void deviceConnected(String location) {
		if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("DEVICE_CONNECT_CONFIRMATION"), LanguageTool.getString("DEVICE_DETECTED")) == JOptionPane.OK_OPTION) {
			VisualHandler.getInstance().showProgressBar(true, null);
			this.retrieveDevice(new File(location));
		}
	}

	/**
	 * Called when a device monitor detects a device disconnection.
	 * 
	 * @param location
	 *            the location
	 */
	@Override
	public void deviceDisconnected(String location) {
		VisualHandler.getInstance().showMessage(LanguageTool.getString("DEVICE_DISCONNECTION_DETECTED"));
		disconnectDevice();

		// Start device connection monitor
		DeviceConnectionMonitor.startMonitor();
	}

	/**
	 * Disconnect device.
	 */

	public void disconnectDevice() {
		// Persist device metadata
		ApplicationDataHandler.getInstance().persistDeviceCache(deviceId, deviceRepository);

		VisualHandler.getInstance().hideDeviceInfoOnStatusBar();

		List<Integer> songsToRemove = new ArrayList<Integer>();
		for (int i = 0; i < PlayListHandler.getInstance().getCurrentPlayList(true).size(); i++) {
			if (PlayListHandler.getInstance().getCurrentPlayList(true).get(i) instanceof AudioFile) {
				AudioFile song = (AudioFile) PlayListHandler.getInstance().getCurrentPlayList(true).get(i);
				if (song.getFile().getPath().startsWith(this.getDeviceRepository().getFolders().get(0).getPath())) {
					songsToRemove.add(i);
				}
			}
		}
		int[] indexes = new int[songsToRemove.size()];
		for (int i = 0; i < songsToRemove.size(); i++) {
			indexes[i] = songsToRemove.get(i);
		}

		if (indexes.length > 0) {
			//			PlayListTable table = VisualHandler.getInstance().getPlayListTable();
			//			((PlayListTableModel) table.getModel()).removeAudioObjects(indexes);
			VisualHandler.getInstance().getPlayListPanel().getPlayListTable().getSelectionModel().setSelectionInterval(-1, -1);
			PlayListHandler.getInstance().removeAudioObjects(indexes);
		}

		deviceRepository = null;
		notifyFinishRefresh(null);
		logger.info(LogCategories.REPOSITORY, "Device disconnected");

		Actions.getAction(ConnectDeviceAction.class).setEnabled(true);
		Actions.getAction(RefreshDeviceAction.class).setEnabled(false);
		Actions.getAction(DisconnectDeviceAction.class).setEnabled(false);

		// Unregister device to search in
		SearchHandler.getInstance().unregisterSearchableObject(DeviceSearchableObject.getInstance());
	}

	/**
	 * Called when closing application
	 */
	public void applicationFinish() {
		if (isDeviceConnected()) {
			// Persist device metadata
			ApplicationDataHandler.getInstance().persistDeviceCache(deviceId, deviceRepository);
		}
	}

	/**
	 * Gets the device artist and album structure.
	 * 
	 * @return the device artist and album structure
	 */
	public Map<String, Artist> getDeviceArtistAndAlbumStructure() {
		if (deviceRepository != null) {
			return deviceRepository.getStructure().getTreeStructure();
		}
		return new HashMap<String, Artist>();
	}

	/**
	 * Gets the device's genre structure
	 * 
	 * @return
	 */
	public Map<String, Genre> getDeviceGenreStructure() {
		if (deviceRepository != null) {
			return deviceRepository.getStructure().getGenreStructure();
		}
		return new HashMap<String, Genre>();
	}

	/**
	 * Returns a string with total and free space for a dir.
	 * 
	 * @return the device data
	 */
	private String getDeviceData() {
		if (deviceRepository != null) {
			int songs = deviceRepository.countFiles();
			File dir = deviceRepository.getFolders().get(0);
			return StringUtils.getString(Integer.toString(songs), " ", LanguageTool.getString("SONGS"), "  (", LanguageTool.getString("FREE_SPACE"), ": ", StringUtils
					.fromByteToMegaOrGiga(dir.getFreeSpace()), ")");
		}
		return null;
	}

	/**
	 * Gets the device folder structure.
	 * 
	 * @return the device folder structure
	 */
	public Map<String, Folder> getDeviceFolderStructure() {
		if (deviceRepository != null) {
			return deviceRepository.getStructure().getFolderStructure();
		}
		return new HashMap<String, Folder>();
	}

	/**
	 * Gets the device repository.
	 * 
	 * @return the device repository
	 */
	public Repository getDeviceRepository() {
		return deviceRepository;
	}

	/**
	 * Gets the device songs.
	 * 
	 * @return the device songs
	 */
	public List<AudioFile> getDeviceSongs() {
		// TODO Rename this method to getFilesList as in RepositoryHandler
		if (deviceRepository != null) {
			return deviceRepository.getAudioFilesList();
		}
		return new ArrayList<AudioFile>();
	}

	/**
	 * Checks if is device connected.
	 * 
	 * @return true, if is device connected
	 */
	public boolean isDeviceConnected() {
		return deviceRepository != null;
	}

	/**
	 * Checks if given file is in the device path.
	 * 
	 * @param path
	 *            Absolute path of the file
	 * 
	 * @return true if file is in device, false otherwise
	 */
	public boolean isDevicePath(String path) {
		if (path.contains(devicePath.toString())) {
			return true;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyCurrentPath(java.lang.String)
	 */
	@Override
	public void notifyCurrentPath(String path) {
		// Nothing to do
	}

	/**
	 * Notify device reload.
	 * 
	 * @param loader
	 *            the loader
	 */
	private void notifyDeviceReload(RepositoryLoader loader) {
		if (ControllerProxy.getInstance().getNavigationController() != null) {
			VisualHandler.getInstance().showProgressBar(false, null);
			ControllerProxy.getInstance().getNavigationController().notifyDeviceReload();

			Actions.getAction(ConnectDeviceAction.class).setEnabled(loader == null);
			Actions.getAction(RefreshDeviceAction.class).setEnabled(loader != null);
			Actions.getAction(DisconnectDeviceAction.class).setEnabled(loader != null);

			// Update status bar info
			if (loader != null) {
				VisualHandler.getInstance().showDeviceInfoOnStatusBar(getDeviceData());
			} else {
				VisualHandler.getInstance().hideDeviceInfoOnStatusBar();
			}

			if (loader != null) {
				// Switch to device view in navigator
				ControllerProxy.getInstance().getNavigationController().setNavigationView(DeviceNavigationView.class.getName());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyFileLoaded()
	 */
	@Override
	public void notifyFileLoaded() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyFilesInRepository(int)
	 */
	@Override
	public void notifyFilesInRepository(int files) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyFinishRead
	 * (net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader)
	 */
	@Override
	public void notifyFinishRead(RepositoryLoader loader) {
		deviceRepository = loader.getRepository();
		logger.info(LogCategories.REPOSITORY, "Device read");
		notifyDeviceReload(loader);

		actionsAfterConnectDevice();
	}

	/**
	 * Actions to execute after connect a device
	 */
	private void actionsAfterConnectDevice() {
		// Start device disconnection monitor
		DeviceDisconnectionMonitor.startMonitor();
		DeviceDisconnectionMonitor.addListener(this);

		// Register device to search in
		SearchHandler.getInstance().registerSearchableObject(DeviceSearchableObject.getInstance());
		// Update device index
		SearchHandler.getInstance().updateSearchIndex(DeviceSearchableObject.getInstance());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyFinishRefresh
	 * (net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader)
	 */
	@Override
	public void notifyFinishRefresh(RepositoryLoader loader) {
		if (loader != null) {
			tempDeviceRepository = loader.getRepository();
		}
		deviceRepository = tempDeviceRepository;
		tempDeviceRepository = null;
		notifyDeviceReload(loader);

		// Update device index
		SearchHandler.getInstance().updateSearchIndex(DeviceSearchableObject.getInstance());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seenet.sourceforge.atunes.kernel.modules.repository.LoaderListener#
	 * notifyRemainingTime(long)
	 */
	@Override
	public void notifyRemainingTime(long time) {
		// Nothing to do
	}

	/**
	 * Refresh device.
	 */
	public void refreshDevice() {
		VisualHandler.getInstance().showProgressBar(true, null);
		logger.info(LogCategories.REPOSITORY, "Refreshing device");
		currentLoader = new RepositoryLoader(deviceRepository.getFolders(), true);
		currentLoader.addRepositoryLoaderListener(this);
		currentLoader.setOldRepository(deviceRepository);
		currentLoader.start();
	}

	/**
	 * Retrieve device.
	 * 
	 * @param path
	 *            the path
	 */
	private void retrieveDevice(File path) {
		logger.info(LogCategories.REPOSITORY, StringUtils.getString("Reading device mounted on ", path));

		setDevicePath(path);

		// Get a unique device identifier
		deviceId = getDeviceIndentificationInfo();

		// Device has been connected before, try to get data from cache
		if (deviceId != null) {
			deviceRepository = ApplicationDataHandler.getInstance().retrieveDeviceCache(deviceId);
			if (deviceRepository != null) {
				refreshDevice();
				return;
			}
		}

		List<File> folders = new ArrayList<File>();
		folders.add(path);
		currentLoader = new RepositoryLoader(folders, false);
		currentLoader.addRepositoryLoaderListener(this);
		currentLoader.start();
	}

	/**
	 * Set the device path.
	 * 
	 * @param path
	 *            Device path (absolute)
	 */
	private void setDevicePath(File path) {
		devicePath = path;
	}

	/**
	 * Gets the file if is in device
	 * 
	 * @param fileName
	 *            the file name
	 * 
	 * @return the file if loaded
	 */
	public AudioFile getFileIfLoaded(String fileName) {
		return deviceRepository == null ? null : deviceRepository.getFile(fileName);
	}

	/**
	 * Tries to read device identifier of device if it has been connect to
	 * aTunes before. If not, writes a new device identifier and returns it
	 * 
	 * @return Unique identifier for connected device
	 */
	private String getDeviceIndentificationInfo() {
		String id = null;
		try {
			File f = new File(StringUtils.getString(devicePath, SystemProperties.fileSeparator, Constants.DEVICE_ID_FILE));
			if (f.exists()) {
				BufferedReader br = new BufferedReader(new FileReader(f));
				id = br.readLine();
				br.close();
			} else {
				// New device identifier is current system time
				id = Long.toString(System.currentTimeMillis());
				BufferedWriter bw = new BufferedWriter(new FileWriter(f));
				bw.append(id);
				bw.newLine();
				bw.flush();
				bw.close();
			}
		} catch (IOException e) {
			logger.error(LogCategories.HANDLER, e);
		}
		return id;
	}
}
