/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.awt.Cursor;
import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;

import net.sourceforge.atunes.gui.views.dialogs.RipCdDialog;
import net.sourceforge.atunes.gui.views.dialogs.RipperProgressDialog;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.RipCDAction;
import net.sourceforge.atunes.kernel.modules.amazon.AmazonAlbum;
import net.sourceforge.atunes.kernel.modules.amazon.AmazonDisc;
import net.sourceforge.atunes.kernel.modules.amazon.AmazonService;
import net.sourceforge.atunes.kernel.modules.cdripper.CdRipper;
import net.sourceforge.atunes.kernel.modules.cdripper.EncoderProgressListener;
import net.sourceforge.atunes.kernel.modules.cdripper.ProgressListener;
import net.sourceforge.atunes.kernel.modules.cdripper.cdda2wav.CdToWavConverter;
import net.sourceforge.atunes.kernel.modules.cdripper.cdda2wav.NoCdListener;
import net.sourceforge.atunes.kernel.modules.cdripper.cdda2wav.model.CDInfo;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.Encoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.FlacEncoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.LameEncoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.Mp4Encoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.NeroAacEncoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.OggEncoder;
import net.sourceforge.atunes.kernel.modules.cdripper.encoders.WavEncoder;
import net.sourceforge.atunes.kernel.utils.PictureExporter;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class RipperHandler.
 */
public class RipperHandler {

	/** The instance. */
	private static RipperHandler instance = new RipperHandler();

	/** The ripper. */
	CdRipper ripper;

	/** The interrupted. */
	volatile boolean interrupted;

	/** The folder created. */
	boolean folderCreated;

	/** The logger. */
	Logger logger = new Logger();

	/** The album cover url. */
	String albumCoverURL;

	/** The quality. */
	private String quality;

	/** The encoder. */
	private String encoder;

	/** The file name pattern. */
	private String fileNamePattern;

	/** executor */
	private ExecutorService executorService = Executors.newSingleThreadExecutor();

	/**
	 * Instantiates a new ripper handler.
	 */
	private RipperHandler() {
		// Nothing to do
	}

	/**
	 * Gets the single instance of RipperHandler.
	 * 
	 * @return single instance of RipperHandler
	 */
	public static RipperHandler getInstance() {
		return instance;
	}

	/**
	 * Add files to existing repository if destiny folder is in repository. This
	 * method is used after an import operation
	 * 
	 * @param files
	 *            the files
	 * @param folder
	 *            the folder
	 */
	private void addFilesToRepositoryAndRefresh(List<File> files, File folder) {
		if (RepositoryHandler.getInstance().isRepository(folder)) {
			String text = StringUtils.getString(LanguageTool.getString("REFRESHING"), "...");
			VisualHandler.getInstance().showProgressBar(true, text);
			RepositoryHandler.getInstance().addAndRefresh(files, folder);
		}
	}

	/**
	 * Cancel process.
	 */
	public void cancelProcess() {
		interrupted = true;
		ripper.stop();
		logger.info(LogCategories.RIPPER, "Process cancelled");
	}

	/**
	 * Fill songs from amazon.
	 * 
	 * @param artist
	 *            the artist
	 * @param album
	 *            the album
	 */
	public void fillSongsFromAmazon(final String artist, final String album) {
		VisualHandler.getInstance().getRipCdDialog().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		VisualHandler.getInstance().getRipCdDialog().getAmazonButton().setEnabled(false);
		new SwingWorker<AmazonAlbum, Void>() {
			@Override
			protected AmazonAlbum doInBackground() throws Exception {
				return AmazonService.getAlbum(artist, album);
			}

			@Override
			protected void done() {
				try {
					if (get() != null) {
						albumCoverURL = get().getImageURL();
						List<String> tracks = new ArrayList<String>();
						for (AmazonDisc disc : get().getDiscs()) {
							tracks.addAll(disc.getTracks());
						}
						VisualHandler.getInstance().getRipCdDialog().updateTrackNames(tracks);
					}
				} catch (InterruptedException e) {
					logger.internalError(e);
				} catch (ExecutionException e) {
					logger.error(LogCategories.RIPPER, e);
				} finally {
					VisualHandler.getInstance().getRipCdDialog().setCursor(Cursor.getDefaultCursor());
					VisualHandler.getInstance().getRipCdDialog().getAmazonButton().setEnabled(true);
				}
			}
		}.execute();
	}

	/**
	 * Gets encoder which was used for ripping CD's.
	 * 
	 * @return Return the encoder used the previous time
	 */
	public String getEncoder() {
		if (encoder != null) {
			return encoder;
		}
		return "OGG";
	}

	/**
	 * Gets the encoder quality.
	 * 
	 * @return the encoder quality
	 */
	public String getEncoderQuality() {
		return quality;
	}

	/**
	 * Returns the filename pattern which is used.
	 * 
	 * @return The filename pattern
	 */
	public String getFileNamePattern() {
		return fileNamePattern;
	}

	/**
	 * Test for avaible encoders and returns an List of the found encoders. In
	 * order to keep the same order as previously, we begin with ogg then mp3
	 * and finally with flac.
	 * 
	 * @return the installed encoders
	 */
	public List<String> getInstalledEncoders() {
		List<String> avaibleEncoders = new ArrayList<String>();
		if (OggEncoder.testTool()) {
			avaibleEncoders.add("OGG");
		}
		if (LameEncoder.testTool()) {
			avaibleEncoders.add("MP3");
		}
		if (Mp4Encoder.testTool()) {
			avaibleEncoders.add("FAAC_MP4");
		}
		if (NeroAacEncoder.testTool()) {
			avaibleEncoders.add("Nero_AAC");
		}
		if (FlacEncoder.testTool()) {
			avaibleEncoders.add("FLAC");
		}
		avaibleEncoders.add("WAV");
		return avaibleEncoders;
	}

	/**
	 * Controlls the import process for ripping audio CD's.
	 * 
	 * @param folder
	 *            The folder where the files should be saved
	 * @param artist
	 *            Artist name (whole CD)
	 * @param album
	 *            Album name
	 * @param year
	 *            Release year
	 * @param genre
	 *            Album genre
	 * @param tracks
	 *            List of the track numbers
	 * @param trckNames
	 *            List of the track names
	 * @param format
	 *            Format in which the files should converted
	 * @param quality1
	 *            Quality setting to be used
	 * @param artistNames
	 *            the artist names
	 * @param composerNames
	 *            the composer names
	 */
	void importSongs(String folder, final String artist, final String album, final int year, final String genre, final List<Integer> tracks, final List<String> trckNames, final List<String> artistNames, final List<String> composerNames, final String format, final String quality1) {
		// Disable import cd option in menu
		Actions.getAction(RipCDAction.class).setEnabled(false);

		final File folderFile = new File(folder);
		if (!folderFile.exists()) {
			if (folderFile.mkdirs()) {
				folderCreated = true;
			} else {
				logger.error(LogCategories.RIPPER, "Folder could not be created");
				return;
			}
		}

		// Prepares commands for the encoder
		Encoder encoder;

		// Ogg encoder
		if (format.equals("OGG")) {
			encoder = new OggEncoder();
		} else if (format.equals("MP3")) {
			encoder = new LameEncoder();
		} else if (format.equals("FLAC")) {
			encoder = new FlacEncoder();
		} else if (format.equals("FAAC_MP4")) {
			encoder = new Mp4Encoder();
		} else if (format.equals("Nero_AAC")) {
			encoder = new NeroAacEncoder();
		} else {
			encoder = new WavEncoder();
		}
		encoder.setQuality(quality1);
		ripper.setEncoder(encoder);
		ripper.setArtist(artist);
		ripper.setAlbum(album);
		ripper.setYear(year);
		ripper.setGenre(genre);
		ripper.setFileNamePattern(fileNamePattern);

		final RipperProgressDialog dialog = VisualHandler.getInstance().getRipperProgressDialog();

		// Get image from amazon if necessary
		if (albumCoverURL != null) {
			Image cover = AmazonService.getAmazonImage(albumCoverURL);
			dialog.setCover(cover);
			savePicture(cover, folderFile, artist, album);
		}

		dialog.setArtistAndAlbum(artist, album);

		dialog.setTotalProgressBarLimits(0, tracks.size());
		dialog.setDecodeProgressBarLimits(0, 100);
		dialog.setEncodeProgressBarLimits(0, 100);

		final List<File> filesImported = new ArrayList<File>();

		ripper.setDecoderListener(new ProgressListener() {
			@Override
			public void notifyFileFinished(File f) {
				// Nothing to do
			}

			@Override
			public void notifyProgress(int percent) {
				dialog.setDecodeProgressValue(percent);
				if (percent > 0) {
					dialog.setDecodeProgressValue(StringUtils.getString(percent, "%"));
				} else {
					dialog.setDecodeProgressValue("");
				}
			}
		});

		ripper.setEncoderListener(new EncoderProgressListener() {
			@Override
			public void notifyFileFinished(File f) {
				// Nothing to do
			}

			@Override
			public void notifyProgress(int percent) {
				dialog.setEncodeProgressValue(percent);
				if (!(percent < 0)) {
					dialog.setEncodeProgressValue(StringUtils.getString(percent, "%"));
				}
			}
		});

		ripper.setTotalProgressListener(new ProgressListener() {
			@Override
			public void notifyFileFinished(File file) {
				filesImported.add(file);
			}

			@Override
			public void notifyProgress(int value) {
				dialog.setTotalProgressValue(value);
				dialog.setTotalProgressValue(StringUtils.getString(value, " / ", tracks.size()));
				dialog.setDecodeProgressValue(0);
				dialog.setDecodeProgressValue(StringUtils.getString(0, "%"));
				dialog.setEncodeProgressValue(0);
				dialog.setEncodeProgressValue(StringUtils.getString(0, "%"));
			}
		});

		new SwingWorker<Boolean, Void>() {
			@Override
			protected Boolean doInBackground() throws Exception {
				return ripper.ripTracks(tracks, trckNames, folderFile, artistNames, composerNames);
			}

			@Override
			protected void done() {
				notifyFinishImport(filesImported, folderFile);
				// Enable import cd option in menu
				Actions.getAction(RipCDAction.class).setEnabled(true);
			}
		}.execute();

		dialog.setVisible(true);
	}

	/**
	 * Notify finish import.
	 * 
	 * @param filesImported
	 *            the files imported
	 * @param folder
	 *            the folder
	 */
	public void notifyFinishImport(final List<File> filesImported, final File folder) {
		VisualHandler.getInstance().getRipperProgressDialog().setVisible(false);
		if (interrupted) { // If process is interrupted delete all imported files
			Runnable deleter = new Runnable() {
				@Override
				public void run() {
					for (File f : filesImported) {
						f.delete();
					}

					// Wait two seconds to assure filesImported are deleted
					try {
						Thread.sleep(2000);
					} catch (InterruptedException e) {
						logger.internalError(e);
					}
					if (folderCreated) {
						folder.delete();
					}
				}
			};
			executorService.submit(deleter);
		} else {
			addFilesToRepositoryAndRefresh(filesImported, folder);
		}
	}

	/**
	 * Save picture.
	 * 
	 * @param image
	 *            the image
	 * @param path
	 *            the path
	 * @param artist
	 *            the artist
	 * @param album
	 *            the album
	 */
	private void savePicture(Image image, File path, String artist, String album) {
		String imageFileName = StringUtils.getString(path.getAbsolutePath(), SystemProperties.fileSeparator, artist, "_", album, "_Cover.png");
		try {
			PictureExporter.savePicture(image, imageFileName);
		} catch (IOException e) {
			logger.internalError(e);
		}
	}

	/**
	 * Sets the encoder.
	 * 
	 * @param encoder
	 *            the new encoder
	 */
	public void setEncoder(String encoder) {
		this.encoder = encoder;
	}

	/**
	 * Sets the encoder quality.
	 * 
	 * @param quality
	 *            the new encoder quality
	 */
	public void setEncoderQuality(String quality) {
		this.quality = quality;
	}

	/**
	 * Sets the used filename pattern.
	 * 
	 * @param fileNamePattern
	 *            The filename pattern used
	 */
	public void setFileNamePattern(String fileNamePattern) {
		this.fileNamePattern = fileNamePattern;
	}

	/**
	 * Start cd ripper.
	 */
	public void startCdRipper() {
		interrupted = false;
		final RipCdDialog dialog = VisualHandler.getInstance().getRipCdDialog();
		VisualHandler.getInstance().showIndeterminateProgressDialog("");

		SwingWorker<CDInfo, Void> getCdInfoAndStartRipping = new SwingWorker<CDInfo, Void>() {
			@Override
			protected CDInfo doInBackground() throws Exception {
				if (!testTools()) {
					return null;
				}
				ripper = new CdRipper();
				ripper.setNoCdListener(new NoCdListener() {
					@Override
					public void noCd() {
						logger.error(LogCategories.RIPPER, "No cd inserted");
						interrupted = true;
						SwingUtilities.invokeLater(new Runnable() {
							@Override
							public void run() {
								VisualHandler.getInstance().hideIndeterminateProgressDialog();
								VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("NO_CD_INSERTED"));
							}
						});
					}
				});
				return ripper.getCDInfo();
			}

			@Override
			protected void done() {
				VisualHandler.getInstance().hideIndeterminateProgressDialog();
				CDInfo cdInfo;
				try {
					cdInfo = get();
					if (cdInfo != null) {
						ControllerProxy.getInstance().getRipCdDialogController().showCdInfo(cdInfo, RepositoryHandler.getInstance().getPathForNewAudioFilesRipped());
						if (!ControllerProxy.getInstance().getRipCdDialogController().isCancelled()) {
							String artist = ControllerProxy.getInstance().getRipCdDialogController().getArtist();
							String album = ControllerProxy.getInstance().getRipCdDialogController().getAlbum();
							int year = ControllerProxy.getInstance().getRipCdDialogController().getYear();
							String genre = ControllerProxy.getInstance().getRipCdDialogController().getGenre();
							String folder = ControllerProxy.getInstance().getRipCdDialogController().getFolder();
							List<Integer> tracks = dialog.getTracksSelected();
							List<String> trckNames = dialog.getTrackNames();
							List<String> artistNames = dialog.getArtistNames();
							List<String> composerNames = dialog.getComposerNames();
							setEncoder(dialog.getFormat().getSelectedItem().toString());
							setEncoderQuality(dialog.getQuality());
							setFileNamePattern(dialog.getFileNamePattern());
							ControllerProxy.getInstance().getRipCdDialogController().setEncoderSettingChanged(false);
							importSongs(folder, artist, album, year, genre, tracks, trckNames, artistNames, composerNames, dialog.getFormat().getSelectedItem().toString(), dialog
									.getQuality());
						} else {
							setEncoder(dialog.getFormat().getSelectedItem().toString());
							setEncoderQuality(dialog.getQuality());
							setFileNamePattern(dialog.getFileNamePattern());
							ControllerProxy.getInstance().getRipCdDialogController().setEncoderSettingChanged(false);
						}
					}
				} catch (InterruptedException e) {
					VisualHandler.getInstance().getRipCdDialog().setVisible(false);
					logger.internalError(e);
				} catch (ExecutionException e) {
					VisualHandler.getInstance().getRipCdDialog().setVisible(false);
					logger.internalError(e);
				}
			}
		};
		getCdInfoAndStartRipping.execute();
	}

	/**
	 * Test the presence of cdda2wav/icedax. Calls the test function from
	 * Cdda2wav.java
	 * 
	 * @return Returns true if cdda2wav/icedax is present, false otherwise
	 */
	boolean testTools() {
		if (!CdToWavConverter.testTool()) {
			logger.error(LogCategories.RIPPER, "Error testing \"cdda2wav\" or \"cdparanoia\". Check program is installed");
			VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("CDDA2WAV_NOT_FOUND"));
			return false;
		}
		return true;
	}
}
