/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.navigator;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.gui.ColorDefinitions;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel.Property;
import net.sourceforge.atunes.gui.views.controls.DragSourceTree;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.AddToPlayListAction;
import net.sourceforge.atunes.kernel.actions.PlayNowAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromFavoritesAction;
import net.sourceforge.atunes.kernel.actions.SetAsPlayListAction;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationController.ViewMode;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;

import org.jvnet.substance.api.renderers.SubstanceDefaultTreeCellRenderer;

public class FavoritesNavigationView extends NavigationView {

	private JTree favoritesTree;

	/** The favorite menu. */
	private JPopupMenu favoriteMenu;

	@Override
	public ImageIcon getIcon() {
		return ImageLoader.FAVORITE;
	}

	@Override
	public String getTitle() {
		return LanguageTool.getString("FAVORITES");
	}

	@Override
	public String getTooltip() {
		return LanguageTool.getString("FAVORITES_TAB_TOOLTIP");
	}

	@Override
	public JTree getTree() {
		if (favoritesTree == null) {
			favoritesTree = new DragSourceTree(new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("FAVORITES"))));
			favoritesTree.setToggleClickCount(0);
			favoritesTree.setCellRenderer(getTreeRenderer());
		}
		return favoritesTree;
	}

	@Override
	public boolean isAudioObjectsFromNodeNeedSort() {
		return true;
	}

	@Override
	public JPopupMenu getTreePopupMenu() {
		if (favoriteMenu == null) {
			favoriteMenu = new JPopupMenu();
			favoriteMenu.add(new JMenuItem(Actions.getAction(AddToPlayListAction.class)));
			favoriteMenu.add(new JMenuItem(Actions.getAction(SetAsPlayListAction.class)));
			favoriteMenu.add(new JMenuItem(Actions.getAction(PlayNowAction.class)));
			favoriteMenu.add(new JMenuItem(Actions.getAction(RemoveFromFavoritesAction.class)));
		}
		return favoriteMenu;
	}

	@Override
	public Map<String, ?> getViewData(ViewMode viewMode) {
		Map<String, Map<?, ?>> data = new HashMap<String, Map<?, ?>>();
		data.put("ARTISTS", FavoritesHandler.getInstance().getFavoriteArtistsInfo());
		data.put("ALBUMS", FavoritesHandler.getInstance().getFavoriteAlbumsInfo());
		return data;
	}

	@Override
	@SuppressWarnings("unchecked")
	protected void refreshTree(ViewMode viewMode, String treeFilter) {
		debug("Refreshing " + this.getClass().getName());

		int[] rowSelected = getTree().getSelectionRows();

		Map<String, ?> data = getViewData(viewMode);

		DefaultTreeModel treeModel = (DefaultTreeModel) getTree().getModel();

		DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();
		root.setUserObject(LanguageTool.getString("FAVORITES"));
		root.removeAllChildren();

		DefaultMutableTreeNode artistsNode = new DefaultMutableTreeNode();
		artistsNode.setUserObject(LanguageTool.getString("ARTISTS"));
		addArtistNodes(artistsNode, (Map<String, Artist>) data.get("ARTISTS"));
		root.add(artistsNode);

		DefaultMutableTreeNode albumsNode = new DefaultMutableTreeNode();
		albumsNode.setUserObject(LanguageTool.getString("ALBUMS"));
		addAlbumNodes(albumsNode, (Map<String, Album>) data.get("ALBUMS"));
		root.add(albumsNode);

		DefaultMutableTreeNode songsNode = new DefaultMutableTreeNode();
		songsNode.setUserObject(LanguageTool.getString("SONGS"));
		root.add(songsNode);

		treeModel.reload();

		getTree().expandPath(new TreePath(artistsNode.getPath()));
		getTree().expandPath(new TreePath(albumsNode.getPath()));

		getTree().expandRow(0);
		getTree().setSelectionRow(rowSelected != null ? rowSelected[0] : 0);
	}

	@Override
	public List<AudioObject> getAudioObjectForTreeNode(DefaultMutableTreeNode node, ViewMode viewMode, String treeFilter) {
		List<AudioObject> songs = null;

		if (node.isRoot()) {
			songs = new ArrayList<AudioObject>();
			songs.addAll(RepositoryHandler.getInstance().getAudioFilesForArtists(FavoritesHandler.getInstance().getFavoriteArtistsInfo()));
			songs.addAll(RepositoryHandler.getInstance().getAudioFilesForAlbums(FavoritesHandler.getInstance().getFavoriteAlbumsInfo()));
			songs.addAll(FavoritesHandler.getInstance().getFavoriteSongsInfo().values());
		} else {
			if (node.getUserObject() instanceof TreeObject) {
				songs = ((TreeObject) node.getUserObject()).getAudioObjects();
			} else {
				songs = new ArrayList<AudioObject>();
				if (node.getUserObject().toString().equals(LanguageTool.getString("ARTISTS"))) {
					songs.addAll(RepositoryHandler.getInstance().getAudioFilesForArtists(FavoritesHandler.getInstance().getFavoriteArtistsInfo()));
				} else if (node.getUserObject().toString().equals(LanguageTool.getString("ALBUMS"))) {
					songs.addAll(RepositoryHandler.getInstance().getAudioFilesForAlbums(FavoritesHandler.getInstance().getFavoriteAlbumsInfo()));
				} else {
					songs.addAll(new ArrayList<AudioFile>(FavoritesHandler.getInstance().getFavoriteSongsInfo().values()));
				}
			}
		}
		return songs;
	}

	/**
	 * Adds the album nodes.
	 * 
	 * @param root
	 *            the root
	 * @param albums
	 *            the albums
	 */
	private static void addAlbumNodes(DefaultMutableTreeNode root, Map<String, Album> albums) {
		List<String> albumsNamesList = new ArrayList<String>(albums.keySet());
		Collections.sort(albumsNamesList);

		for (int i = 0; i < albumsNamesList.size(); i++) {
			Album album = albums.get(albumsNamesList.get(i));
			DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
			root.add(albumNode);
		}
	}

	/**
	 * Adds the artist nodes.
	 * 
	 * @param root
	 *            the root
	 * @param artists
	 *            the artists
	 */
	private static void addArtistNodes(DefaultMutableTreeNode root, Map<String, Artist> artists) {
		List<String> artistNamesList = new ArrayList<String>(artists.keySet());
		Collections.sort(artistNamesList);

		for (int i = 0; i < artistNamesList.size(); i++) {
			Artist artist = artists.get(artistNamesList.get(i));
			DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
			addAlbumNodes(artistNode, artist.getAlbums());
			root.add(artistNode);
		}
	}

	@Override
	public Class<?> getNavigatorTableColumnClass(int columnIndex) {
		switch (columnIndex) {
		case 0:
			return Property.class;
		case 1:
			return String.class;
		case 2:
			return Long.class;
		default:
			return Long.class;
		}
	}

	@Override
	public int getNavigatorTableColumnCount() {
		return 3;
	}

	@Override
	public String getNavigatorTableColumnName(int columnIndex) {
		switch (columnIndex) {
		case 0:
			return "";
		case 1:
			return LanguageTool.getString("TITLE");
		case 2:
			return LanguageTool.getString("DURATION");
		default:
			return "";
		}
	}

	@Override
	public Object getNavigatorTableValueAt(AudioObject audioObject, int columnIndex) {
		if (audioObject instanceof AudioFile) {
			if (columnIndex == 0) {
				return Kernel.getInstance().state.isShowFavoritesInNavigator() && FavoritesHandler.getInstance().getFavoriteSongs().contains(audioObject) ? Property.FAVORITE
						: Property.NO_PROPERTIES;
			} else if (columnIndex == 1) {
				return audioObject.getTitleOrFileName();
			} else if (columnIndex == 2) {
				return audioObject.getDuration();
			}
		}
		return "";
	}

	@Override
	public int getNavigatorTableMaxWidthForColumn(int columnIndex) {
		if (columnIndex == 0) {
			return 20;
		} else if (columnIndex == 2) {
			return 50;
		}
		return -1;
	}

	@Override
	public boolean isViewModeSupported() {
		return false;
	}

	@Override
	public List<AudioObject> filterNavigatorTable(List<AudioObject> audioObjects, String filter) {
		return null;
	}

	@Override
	public boolean isNavigatorTableFilterSupported() {
		return false;
	}

	@Override
	public void updateTreePopupMenuWithTreeSelection(MouseEvent e) {
		TreePath path = getTree().getSelectionPath();
		if (path != null) {
			Object obj = ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
			if (obj instanceof Album) {
				Object[] objs = ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObjectPath();
				boolean foundArtist = false;
				for (Object element : objs) {
					if (element instanceof Artist) {
						foundArtist = true;
						break;
					}
				}
				Actions.getAction(RemoveFromFavoritesAction.class).setEnabled(!foundArtist);
			} else {
				Actions.getAction(RemoveFromFavoritesAction.class).setEnabled(obj instanceof TreeObject);
			}
		}
		Actions.getAction(PlayNowAction.class).setEnabled(false);
	}

	@Override
	public void updateTablePopupMenuWithTableSelection(JTable table, MouseEvent e) {
		Actions.getAction(PlayNowAction.class).setEnabled(table.getSelectedRowCount() == 1);

		if (table.getSelectedRowCount() > 0) {
			if (getTree().getSelectionCount() == 1) {
				Object obj = ((DefaultMutableTreeNode) getTree().getSelectionPath().getLastPathComponent()).getUserObject();
				if (obj instanceof String && obj.equals(LanguageTool.getString("SONGS"))) {
					Actions.getAction(RemoveFromFavoritesAction.class).setEnabled(true);
				} else {
					Actions.getAction(RemoveFromFavoritesAction.class).setEnabled(false);
				}
			} else {
				Actions.getAction(RemoveFromFavoritesAction.class).setEnabled(false);
			}
		}
	}

	@Override
	protected TreeCellRenderer getTreeRenderer() {
		return new SubstanceDefaultTreeCellRenderer() {
			private static final long serialVersionUID = 2880969518313022116L;

			@Override
			public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
				Object content = node.getUserObject();

				JLabel label = (JLabel) super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
				// Set custom icon for every type of node: artist, album, song...
				if (content instanceof Artist) {
					label.setIcon(ImageLoader.ARTIST);
				} else if (content instanceof Album) {
					label.setIcon(ImageLoader.ALBUM);
				} else if (content instanceof String) {
					String str = (String) content;
					if (str.equals(LanguageTool.getString("ARTISTS"))) {
						label.setIcon(ImageLoader.ARTIST);
					} else if (str.equals(LanguageTool.getString("ALBUMS"))) {
						label.setIcon(ImageLoader.ALBUM);
					} else if (str.equals(LanguageTool.getString("SONGS"))) {
						label.setIcon(ImageLoader.AUDIO_FILE);
					} else {
						label.setIcon(ImageLoader.FAVORITE);
					}
				}

				if (value.toString() != null) {
					if (value.toString().equals(LanguageTool.getString("UNKNOWN_ARTIST")) || value.toString().equals(LanguageTool.getString("UNKNOWN_ALBUM"))) {
						label.setForeground(ColorDefinitions.GENERAL_UNKNOWN_ELEMENT_FOREGROUND_COLOR);
					}
				}

				return label;
			}
		};
	}
}
