/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.navigator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.model.TreeObject;

/**
 * The Class RefreshUtils.
 */
public class RefreshUtils {

	/**
	 * Adds the folder nodes.
	 * 
	 * @param folders
	 *            the folders
	 * @param node
	 *            the node
	 * @param currentFilter
	 *            the current filter
	 * @param comparator
	 *            the comparator for node sorting
	 */
	static void addFolderNodes(Map<String, ?> folders, DefaultMutableTreeNode node, String currentFilter, Comparator<String> comparator) {
		List<String> folderNamesList = new ArrayList<String>(folders.keySet());
		Collections.sort(folderNamesList, comparator);

		for (int i = 0; i < folderNamesList.size(); i++) {
			Folder f = (Folder) folders.get(folderNamesList.get(i));
			if (node.isRoot() || currentFilter == null || f.getName().toUpperCase().contains(currentFilter.toUpperCase())) {
				DefaultMutableTreeNode child = new DefaultMutableTreeNode(f);
				node.add(child);
				addFolderNodes(f.getFolders(), child, node.isRoot() ? currentFilter : null, comparator);
			}
		}
	}

	/**
	 * Returns all TreeObject instances selected in a tree
	 * 
	 * @param tree
	 * @return
	 */
	static List<TreeObject> getTreeObjectsSelected(JTree tree) {
		// Get objects selected before refreshing tree
		List<TreeObject> objectsSelected = new ArrayList<TreeObject>();
		TreePath[] pathsSelected = tree.getSelectionPaths();

		// If any node was selected
		if (pathsSelected != null) {
			for (TreePath pathSelected : pathsSelected) {
				Object obj = ((DefaultMutableTreeNode) pathSelected.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsSelected.add((TreeObject) obj);
				}
			}
		}

		return objectsSelected;
	}

	/**
	 * Returns all TreeObject instances expanded in a tree
	 * 
	 * @param tree
	 * @param root
	 * @return
	 */
	static List<TreeObject> getTreeObjectsExpanded(JTree tree, DefaultMutableTreeNode root) {
		// Get objects expanded before refreshing tree
		List<TreeObject> objectsExpanded = new ArrayList<TreeObject>();
		Enumeration<TreePath> enume = tree.getExpandedDescendants(new TreePath(root.getPath()));

		// If any node was expanded
		if (enume != null) {
			while (enume.hasMoreElements()) {
				TreePath p = enume.nextElement();
				Object obj = ((DefaultMutableTreeNode) p.getLastPathComponent()).getUserObject();
				if (obj instanceof TreeObject) {
					objectsExpanded.add((TreeObject) obj);
				}
			}
		}

		return objectsExpanded;
	}

	/**
	 * Expands a list of nodes of a tree
	 * 
	 * @param tree
	 * @param nodesToExpand
	 */
	static void expandNodes(JTree tree, List<DefaultMutableTreeNode> nodesToExpand) {
		for (DefaultMutableTreeNode node : nodesToExpand) {
			tree.expandPath(new TreePath(node.getPath()));
		}
	}

	/**
	 * Selects a list of nodes of a tree
	 * 
	 * @param tree
	 * @param nodesToSelect
	 */
	static void selectNodes(JTree tree, List<DefaultMutableTreeNode> nodesToSelect) {
		if (nodesToSelect.isEmpty()) {
			tree.setSelectionRow(0);
		} else {
			TreePath[] pathsToSelect = new TreePath[nodesToSelect.size()];
			int i = 0;
			for (DefaultMutableTreeNode node : nodesToSelect) {
				pathsToSelect[i++] = new TreePath(node.getPath());
			}
			tree.setSelectionPaths(pathsToSelect);
		}
	}

	/**
	 * Recursive method to find nodes, based on whether user objects are present
	 * in given set
	 * 
	 * @param rootNode
	 * @param userObjects
	 * @return list of nodes
	 */
	static List<DefaultMutableTreeNode> getNodes(DefaultMutableTreeNode rootNode, List<TreeObject> userObjects) {
		List<DefaultMutableTreeNode> result = new ArrayList<DefaultMutableTreeNode>();

		if (userObjects.contains(rootNode.getUserObject())) {
			result.add(rootNode);
		}
		for (int i = 0; i < rootNode.getChildCount(); i++) {
			result.addAll(getNodes((DefaultMutableTreeNode) rootNode.getChildAt(i), userObjects));
		}
		return result;
	}

}
