/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.process;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sourceforge.atunes.gui.views.dialogs.ProgressDialog;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.utils.StringUtils;

public abstract class AudioFileTransferProcess extends Process {

	/** 
	 * The files to be transferred. 
	 */
	protected List<AudioFile> filesToTransfer;

	/**
	 * List of files transferred. Used if process is canceled to delete these files
	 */
	protected List<File> filesTransferred;
	
	protected AudioFileTransferProcess(List<AudioFile> files) {
		this.filesToTransfer = files;
		this.filesTransferred = new ArrayList<File>();
	}
	
	@Override
	protected long getProcessSize() {
		// Get size of files
		long totalBytes = 0;
		for (AudioFile file : this.filesToTransfer) {
			totalBytes = totalBytes + file.getFile().length();
		}
		return totalBytes;
	}
	
	@Override
	protected ProgressDialog getProgressDialog() {
		if (progressDialog == null) {
			// Use a TransferProgressDialog
			progressDialog = VisualHandler.getInstance().getNewTransferProgressDialog(getProgressDialogTitle(), owner);
			progressDialog.setInfoText(getProgressDialogInformation());
			progressDialog.setCurrentProgress(0);
			progressDialog.setProgressBarValue(0);
			progressDialog.addCancelButtonActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					cancelProcess();
					progressDialog.disableCancelButton();
				}
			});
		}
		return progressDialog;
	}

	@Override
	protected boolean runProcess() {
		boolean errors = false;
		try {
			File destination = new File(getDestination());
			long bytesTransferred = 0;
			addInfoLog(StringUtils.getString("Transferring ", this.filesToTransfer.size(), " files to ", destination));
			for (Iterator<AudioFile> it = this.filesToTransfer.iterator(); it.hasNext() && !cancel;) {
				AudioFile file = it.next();
				File transferredFile = transferAudioFile(destination, file);
				filesTransferred.add(transferredFile);
				// Add size to bytes transferred
				bytesTransferred += file.getFile().length();
				setCurrentProgress(bytesTransferred);
			}		
			addInfoLog("Transfer process done");
		} catch (Exception e) {
			addErrorLog(e);
			errors = true;
		}
		return !errors;
	}
	
	@Override
	protected void runCancel() {
		// Remove all transferred files
		for (File f : this.filesTransferred) {
			f.delete();
		}
	}

	/**
	 * Return destination of all files to be transferred
	 * @return
	 */
	protected abstract String getDestination();
	
	/**
	 * Transfers a file to a destination
	 * @param destination
	 * @param file
	 * @return
	 * @throws IOException
	 */
	protected abstract File transferAudioFile(File destination, AudioFile file) throws IOException;
}
