/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.repository;

import java.io.File;
import java.io.IOException;
import java.util.List;

import net.sourceforge.atunes.kernel.modules.process.AudioFileTransferProcess;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.FileNameUtils;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

import org.apache.commons.io.FileUtils;

/**
 * Exports (song) files to a partition/device and checks if filename is valid.
 */
public class ExportFilesProcess extends AudioFileTransferProcess {

	/** The Constant ALL_EXPORT. */
	public static final int ALL_EXPORT = 1;

	/** The Constant SELECTION_EXPORT. */
	public static final int SELECTION_EXPORT = 2;

	/** The Constant FAVORITES_EXPORT. */
	public static final int FAVORITES_EXPORT = 3;

	/** The Constant FLAT_STRUCTURE. */
	public static final int FLAT_STRUCTURE = 1;

	/** The Constant ARTIST_STRUCTURE. */
	public static final int ARTIST_STRUCTURE = 2;

	/** The Constant FULL_STRUCTURE. */
	public static final int FULL_STRUCTURE = 3;

	/** The structure. */
	private int structure;

	/** The file pattern. */
	private String filePattern;

	/** The path. */
	private String path;

	/**
	 * Exports songs from the repository to an external path, call via
	 * BackgroundExecutor.java . Sets the values
	 * 
	 * @param songs
	 *            List with songs to export
	 * @param structure
	 *            Folder structure, either 1, 2 or 3
	 * @param filePattern
	 *            Filename pattern
	 * @param path
	 *            Path to where the files should be exported
	 */
	public ExportFilesProcess(List<AudioFile> songs, int structure, String filePattern, String path) {
		super(songs);
		this.structure = structure;
		this.filePattern = filePattern;
		this.path = path;
	}

	@Override
	public String getProgressDialogTitle() {
		return LanguageTool.getString("EXPORT_FILES");
	}
	
	/**
	 * Prepares the directory structure in which the song will be written.
	 * 
	 * @param song
	 *            Song to be written
	 * @param destination
	 *            Destination path
	 * @param struct
	 *            structure Folder structure, either 1, 2 or 3
	 * 
	 * @return Returns the directory structure with full path where the file
	 *         will be written
	 */
	public static File getDirectory(AudioFile song, File destination, int struct, boolean isMp3Device) {
		if (struct == FLAT_STRUCTURE) {
			return new File(FileNameUtils.getValidFolderName(destination.toString().replace("\\", "\\\\").replace("$", "\\$"), isMp3Device));
		} else if (struct == ARTIST_STRUCTURE) {
			if (song.getAlbumArtist().isEmpty()) {
				return new File(StringUtils.getString(destination.getAbsolutePath(), SystemProperties.fileSeparator, FileNameUtils.getValidFileName(song.getArtist().replace("\\",
						"\\\\").replace("$", "\\$"), isMp3Device)));
			} else {
				return new File(StringUtils.getString(destination.getAbsolutePath(), SystemProperties.fileSeparator, FileNameUtils.getValidFileName(song.getAlbumArtist().replace(
						"\\", "\\\\").replace("$", "\\$"), isMp3Device)));
			}
		} else {
			// FULL_STRUCTURE
			if (song.getAlbumArtist().isEmpty()) {
				return new File(StringUtils.getString(destination.getAbsolutePath(), SystemProperties.fileSeparator, FileNameUtils.getValidFileName(song.getArtist().replace("\\",
						"\\\\").replace("$", "\\$"))
						+ SystemProperties.fileSeparator + FileNameUtils.getValidFileName(song.getAlbum().replace("\\", "\\\\").replace("$", "\\$"), isMp3Device)));
			} else {
				return new File(StringUtils.getString(destination.getAbsolutePath(), SystemProperties.fileSeparator, FileNameUtils.getValidFileName(song.getAlbumArtist().replace(
						"\\", "\\\\").replace("$", "\\$"))
						+ SystemProperties.fileSeparator + FileNameUtils.getValidFileName(song.getAlbum().replace("\\", "\\\\").replace("$", "\\$"), isMp3Device)));
			}
		}
	}

	@Override
	protected String getDestination() {
		return this.path;
	}
	
	@Override
	protected File transferAudioFile(File destination, AudioFile file) throws IOException {
		File destDir = getDirectory(file, destination, structure, false);
		String newName;
		if (filePattern != null) {
			newName = FileNameUtils.getNewName(filePattern, file);
		}
		else {
			// Files written to mp3 devices do not have any filepattern passed, so only check if
			// if filename is valid
			newName = FileNameUtils.getValidFileName(file.getFile().getName().replace("\\", "\\\\").replace("$", "\\$"), false);
		}

		File destFile = new File(StringUtils.getString(destDir.getAbsolutePath(), SystemProperties.fileSeparator, newName));
		// Now that we (supposedly) have a valid filename write file
		FileUtils.copyFile(file.getFile(), destFile);
		
		return destFile;
	}	
}
