/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.repository.tags.writer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class TagEditionOperations.
 */
public class TagEditionOperations {

	/** The number separator pattern. */
	private static Pattern NUMBER_SEPARATOR_PATTERN = Pattern.compile("[^0-9]+");

	/**
	 * Adds the lyrics.
	 * 
	 * @param files
	 *            the files
	 */
	public static void addLyrics(List<AudioFile> files) {
		SetLyricsProcess process = new SetLyricsProcess(files);
		process.execute();
	}

	/**
	 * Edits the album name.
	 * 
	 * @param files
	 *            the files
	 */
	private static void editAlbumName(List<AudioFile> files) {
		SetAlbumNamesProcess process = new SetAlbumNamesProcess(files);
		process.execute();
	}

	/**
	 * Sets genre based on Last.fm tags
	 * 
	 * @param files
	 *            the files
	 */
	public static void editGenre(List<AudioFile> files) {
		SetGenresProcess process = new SetGenresProcess(files);
		process.execute();
	}

	public static void editCover(List<AudioFile> files) {
		SetCoversProcess process = new SetCoversProcess(files);
		process.execute();
	}

	/**
	 * Sets track number based on file name to an array of files.
	 * 
	 * @param files
	 *            the files
	 */
	public static void editTrackNumber(List<AudioFile> files) {
		/*
		 * Given an array of files, returns a map containing each file and its
		 * track number based on information found on file name.
		 */
		Map<AudioFile, Integer> filesToSet = new HashMap<AudioFile, Integer>();
		for (int j = 0; j < files.size(); j++) {

			// Try to get a number from file name
			String fileName = files.get(j).getNameWithoutExtension();
			String[] aux = NUMBER_SEPARATOR_PATTERN.split(fileName);

			int trackNumber = 0;
			int i = 0;
			while (trackNumber == 0 && i < aux.length) {
				String token = aux[i];
				try {
					trackNumber = Integer.parseInt(token);
				} catch (NumberFormatException e) {
					// Ok, it's not a valid number, skip it
				}
				i++;
			}

			if (trackNumber != 0) {
				filesToSet.put(files.get(j), trackNumber);
			}
		}
		if (!filesToSet.isEmpty()) {
			// Call process
			SetTrackNumberProcess process = new SetTrackNumberProcess(filesToSet);
			process.execute();
		}
	}

	/**
	 * Repair album names.
	 */
	public static void repairAlbumNames() {
		// Show confirmation dialog
		if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("REPAIR_ALBUM_NAMES_MESSAGE"), LanguageTool.getString("REPAIR_ALBUM_NAMES")) != JOptionPane.OK_OPTION) {
			return;
		}

		// Get all repository songs
		List<AudioFile> repositorySongs = RepositoryHandler.getInstance().getAudioFiles();

		// Call album name edit
		editAlbumName(repositorySongs);
	}

	/**
	 * Sets genres on files that don't have correct genre.
	 */
	public static void repairGenres() {
		// Show confirmation dialog
		if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("REPAIR_GENRES_MESSAGE"), LanguageTool.getString("REPAIR_GENRES")) != JOptionPane.OK_OPTION) {
			return;
		}

		// Get all repository songs
		List<AudioFile> repositorySongs = RepositoryHandler.getInstance().getAudioFiles();

		// Call genre edit
		editGenre(repositorySongs);
	}

	/**
	 * Sets track number to files that don't have track number setted.
	 */

	public static void repairTrackNumbers() {
		// Show confirmation dialog
		if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("REPAIR_TRACK_NUMBERS_MESSAGE"), LanguageTool.getString("REPAIR_TRACK_NUMBERS")) != JOptionPane.OK_OPTION) {
			return;
		}

		// Get all repository songs
		List<AudioFile> repositorySongs = RepositoryHandler.getInstance().getAudioFiles();

		// Get songs with empty track number
		List<AudioFile> songsToBeRepaired = new ArrayList<AudioFile>();
		for (AudioFile song : repositorySongs) {
			if (song.getTrackNumber() == 0) {
				songsToBeRepaired.add(song);
			}
		}

		// Call track number edit
		editTrackNumber(songsToBeRepaired);
	}
}
